@@ -62,7 +62,7 @@ Options:
                               value of each parameter explicitly changed by
                               option -p/--param.
   --format FORMAT             Specify an image format for diagram outputs
-                              [default: png].
+                              [default: svg].
 
 Advanced options:
   --seed SEED                 Set a seed value for random numbers. When not
@@ -95,31 +95,30 @@ EMULSION (Epidemiological Multi-Level Simulation framework)
 Contributors and contact:
 -------------------------
 
-    - Sébastien Picault (sebastien.picault@inra.fr)
+    - Sébastien Picault (sebastien.picault@inrae.fr)
     - Yu-Lin Huang
     - Vianney Sicard
     - Sandie Arnoux
     - Gaël Beaunée
-    - Pauline Ezanno (pauline.ezanno@inra.fr)
+    - Pauline Ezanno (pauline.ezanno@inrae.fr)
 
-    BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-    Nantes CS 44307 CEDEX, France
+    INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 
 
 How to cite:
 ------------
 
-    S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-    Sustainability of Complex Epidemiological Models through a Generic
-    Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-    International Joint Conference on Artificial Intelligence (IJCAI),
-    AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+    S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+    P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+    stochastic models in human, animal and plant epidemiology", PLoS
+    Computational Biology 15(9): e1007342. DOI:
+    10.1371/journal.pcbi.1007342
 
 
 License:
 --------
 
-   Copyright 2016 INRA and Univ. Lille
+   Copyright 2016 INRAE and Univ. Lille
 
    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 
@@ -182,7 +181,7 @@ from   emulsion.tools.misc          import load_class
 from   emulsion.tools.plot          import plot_outputs
 from   emulsion.tools.simulation    import MultiSimulation
 
-VERSION = '1.0'
+VERSION = '1.0.12'
 DEFAULT_VERSION = "1.0"
 LICENSE = 'Apache-2.0'
 DEFAULT_LICENSE = "Apache-2.0"
@@ -330,6 +329,9 @@ def run_model(params):
     count_path = Path(params.output_dir, 'counts.csv')
     if count_path.exists():
         count_path.unlink()
+    log_path = Path(params.output_dir, 'log.txt')
+    if log_path.exists():
+        log_path.unlink()
     multi_simu = MultiSimulation(**params)
     # multi_simu.write_dot()
     start = time.perf_counter()
