@@ -11,31 +11,30 @@ Classes and functions for actions.
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -130,13 +129,46 @@ class ValueAction(AbstractAction):
         self.parameter = parameter
         self.delta_t = delta_t
 
+class RecordChangeAction(ValueAction):
+    """RecordChangeAction allows to record how many agents performed an
+    action set. The corresponding value is added to a variable assumed
+    to be defined at the upper level.
+
+    """
+    def __init__(self, parameter=None, model=None, **others):
+        """Create a RecordChangeAction aimed at modifying the specified
+        parameter, (assumed to be a statevar defined at the upper
+        level).
+
+        """
+        super().__init__(**others)
+        self.statevar_name = parameter
+        # print(self)
+
+    def execute_action(self, unit, population=None, agents=None):
+        """Execute the action on the specified unit, with the
+        specified population size.
+
+        """
+        if population is None:
+            population = len(agents)
+        target_agent = unit.upper_level()
+        value = target_agent.statevars[self.statevar_name]\
+                if self.statevar_name in target_agent.statevars else 0
+        target_agent.statevars[self.statevar_name] = value + population
+
+    def __str__(self):
+        return super().__str__() + ' on {}'.format(self.statevar_name)
+    __repr__ = __str__
+
+
 class SetVarAction(ValueAction):
-    """SetVarAction allow to set the variable of the agent.
+    """SetVarAction allows to set the variable of the agent.
 
     """
     def __init__(self, statevar_name=None, parameter=None, model=None, **others):
         """Create a SetVarAction aimed at modifying the specified statevar
-        according to the paramter.
+        according to the parameter.
 
         """
         super().__init__(**others)
@@ -158,7 +190,13 @@ class SetVarAction(ValueAction):
         if agents is None:
             agents = [unit]
         for agent in agents:
-            value = agent.get_model_value(self.parameter)
+            # if the value which has to be affected to the variable is
+            # already a statevar, retrieve its value from the agent
+            if self.parameter in agent.statevars:
+                value = agent.statevars[self.parameter]
+            # otherwise get its value through the model
+            else:
+                value = agent.get_model_value(self.parameter)
             # agent.set_information(self.statevar_name, value)
             agent.statevars[self.statevar_name] = value
 
@@ -293,12 +331,26 @@ class StringAction(AbstractAction):
     __repr__ = __str__
 
 
-class BecomeAction(StringAction):
+class BecomeAction(AbstractAction):
     """A BecomeAction is aimed at making an agent change its state
-    according to a specified prototype an action. It requires a
-    prototype name.
+    according to one ore more specified prototypes.
 
     """
+    def __init__(self, prototypes=[], probas=None, model=None, **others):
+        super().__init__(**others)
+        self.prototype_names = prototypes if isinstance(prototypes, list)\
+                               else [prototypes]
+        if probas is None:
+            probas = [1/len(self.prototype_names)] * len(self.prototype_names)
+        else:
+            probas = probas if isinstance(probas, list) else [probas]
+        assert(0 <= len(self.prototype_names) - len(probas) <= 1)
+        self.probas = [model.add_expression(pr) for pr in probas]
+
+    def __str__(self):
+        return super().__str__() + ' ({!s}, {})'.format(self.prototype_names, self.probas)
+    __repr__ = __str__
+
     def execute_action(self, unit, agents=None, **others):
         """Execute the action in the specified unit. If the `agents` parameter
         is specified (as a list), each agent of this list will execute
@@ -311,7 +363,23 @@ class BecomeAction(StringAction):
         if agents is None:
             agents = [unit]
         for agent in agents:
-            agent.apply_prototype(name=self.parameter, execute_actions=True)
+            protos = list(self.prototype_names)
+            # compute actual values for probabilities
+            proba_values = [agent.get_model_value(prob) for prob in self.probas]
+            total = sum(proba_values)
+            assert(0 <= total <= 1)
+            if total < 1:
+                # add complement
+                proba_values.append(1 - total)
+                # if N-1 probabilities were given for N prototypes, no
+                # problem: the last prototype is getting the 1-total
+                # value. Otherwise, this means that there is a
+                # possibility that no individuals are produced => None
+                if len(proba_values) > len(protos):
+                    protos.append(None)
+            prototype = np.random.choice(protos, p=proba_values)
+            if prototype is not None:
+                agent.apply_prototype(name=prototype, execute_actions=True)
 
 class CloneAction(AbstractAction):
     """A CloneAction produces several copies of the agent with a given
@@ -327,7 +395,7 @@ class CloneAction(AbstractAction):
             probas = [1/len(self.prototype_names)] * len(self.prototype_names)
         else:
             probas = probas if isinstance(probas, list) else [probas]
-        assert(len(self.prototype_names) - len(probas) <= 1)
+        assert(0 <= len(self.prototype_names) - len(probas) <= 1)
         self.amount = model.add_expression(amount)
         self.probas = [model.add_expression(pr) for pr in probas]
 
@@ -380,9 +448,9 @@ class MessageAction(StringAction):
     For instance, 'My state is {.statevars.health_state}' will print
     the current health state of the agent.
 
-    Output is formatted in three comma-separated fields: the time step
-    when the message was produced, the agent speaking, and the message
-    itself.
+    Output is formatted in four comma-separated fields: the simulation
+    ID, the time step when the message was produced, the agent
+    speaking, and the message itself.
 
     """
     def execute_action(self, unit, agents=None, **others):
@@ -395,8 +463,31 @@ class MessageAction(StringAction):
             agents = [unit]
         for agent in agents:
             message = self.parameter.format(agent)
-            print("@{}, {}, {}".format(agent.statevars.step, agent, message))
+            print("${},@{},{},{}".format(agent.get_information('simu_id'), agent.statevars.step, agent, message))
 
+class LogVarsAction(StringAction):
+    """A LogVarsAction is aimed at making an agent print a list of
+    variables into a file. Output is formatted in several
+    comma-separated fields: the simulation ID, the time step when the
+    message was produced, the class and ID of agent speaking, and the
+    values of variables (in the order they were defined in the
+    action).
+
+    """
+    def execute_action(self, unit, agents=None, **others):
+        """Execute the action in the specified unit. If the `agents` parameter
+        is specified (as a list), each agent of this list will execute
+        the action.
+
+        """
+        if agents is None:
+            agents = [unit]
+        for agent in agents:
+            message = '{},{},{},{}'.format(agent.get_information('simu_id'), agent.statevars.step, agent.__class__.__name__, agent.agid)
+            variables = ','.join(['{}={}'.format(varname, agent.get_information(varname))
+                                  for varname in self.l_params])
+            with open(agent.log_path(), 'a') as logfile:
+                logfile.write('{},{}\n'.format(message, variables))
 
 class MethodAction(AbstractAction):
     """A MethodAction is aimed at making an agent perform an action on
@@ -474,10 +565,12 @@ ACTION_DICT = {
     'increase_stoch': StochIncreaseAction,
     'decrease_stoch': StochDecreaseAction,
     'message': MessageAction,
+    'log_vars': LogVarsAction,
     'become': BecomeAction,
     'clone': CloneAction,
     'produce_offspring': CloneAction,
     'action': MethodAction,
     'duration': FunctionAction,
-    'set_var': SetVarAction
+    'set_var': SetVarAction,
+    'record_change': RecordChangeAction,
 }
