@@ -14,31 +14,30 @@ Univ. Lille).
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -275,7 +274,7 @@ class AbstractAgent(object, metaclass=MetaAgent):
             elif name in self.model._values:
                 return self.get_model_value(name)
             else:
-                print(str(self), self._mbr_cache)
+                # print(str(self), self._mbr_cache)
                 raise StateVarNotFoundException(name, self)
 
     def set_information(self, name, value):
@@ -342,7 +341,7 @@ class AbstractAgent(object, metaclass=MetaAgent):
         if do_actions:
             # do on_enter actions associated to the new value
             # print(f'doing on_enter for {state_machine} in {new_state}')
-            self.do_state_actions('on_enter', state_machine, new_state.name)
+            self.do_state_actions('on_enter', state_machine, new_state.name, agents=[self])
 
     # DEPRECATED in >0.9.5
     # def init_time_to_live(self, machine_name, value):
