@@ -14,31 +14,30 @@ Univ. Lille).
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -92,7 +91,8 @@ class EmulsionAgent(AbstractAgent):
     Source/Destination agent.
 
     """
-    def __init__(self, model=None, level=None, name=None, host=None, **others):
+    def __init__(self, model=None, level=None, name=None, host=None,
+                 simulation=None, **others):
         """Initialize the unit with a health state and a name."""
         super().__init__(**others)
         # self.statevars.health_state = health_state
@@ -100,15 +100,16 @@ class EmulsionAgent(AbstractAgent):
         self.level = level
         self._name = name
         self._host = host
+        self.simulation = simulation
         if 'step' not in self.statevars:
             self.statevars.step = 0
+        self.statevars.creation_step = self.statevars.step
         # exchange inbox/outbox
         self._inbox = []
         self._outbox = []
 
-
     def upper_level(self, init=True):
-        """Return the 'upper level' from this agent, i.e. the first host with
+        """Return the 'upper level' for this agent, i.e. the first host with
         a not-None level attribute.
 
         TAG: USER
@@ -117,6 +118,16 @@ class EmulsionAgent(AbstractAgent):
             return self
         return self.get_host().upper_level(init=False)
 
+    def top_level(self):
+        """Return the highest level used in the simulation"""
+        if self.get_host() is None:
+            return self
+        return self.upper_level().top_level()
+
+    def log_path(self):
+        """Return the path to the log file for the current simulation"""
+        return self.top_level().simulation.log_path()
+
     @property
     def name(self):
         """Return the name of the unit. If no name was provided during
@@ -325,7 +336,7 @@ class EmulsionAgent(AbstractAgent):
             # if stochastic uses binomial
             if stochastic:
                 # aggregate probability wrt the time step duration
-                value = aggregate_probabilities([value], self.model.delta_t)
+                value = aggregate_probabilities([value], self.model.delta_t)[0]
                 amount = np.random.binomial(pop_size, value)
             else:
                 # otherwise treat proba as proportion
@@ -334,9 +345,12 @@ class EmulsionAgent(AbstractAgent):
             # if stochastic: use Poisson distribution
             if stochastic:
                 amount = np.random.poisson(value * pop_size * self.model.delta_t)
+                # amount = np.random.binomial(pop_size, 1-np.exp(-value * self.model.delta_t))
             else:
                 # consider rate as a speed
-                amount = value * pop_size * self.model.delta_t
+                # S = S0 exp(rate * dt) => S += S * (exp(rate*dt) - 1)
+                # amount = value * pop_size * self.model.delta_t
+                amount = pop_size * (np.exp(value * self.model.delta_t) - 1)
         else:                   # should not happen !
             amount = None
         return amount
