@@ -12,31 +12,30 @@
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -65,7 +64,7 @@ from   emulsion.agent.comparts   import Compartment
 
 from   emulsion.agent.managers.group_manager  import  GroupManager
 from   emulsion.agent.managers.abstract_process_manager  import  AbstractProcessManager
-
+from   emulsion.agent.exceptions import StateVarNotFoundException
 
 #   _____                                 _   _____
 #  / ____|                               | | |  __ \
@@ -161,7 +160,11 @@ class CompartProcessManager(AbstractProcessManager):
     def add_new_population(self, process_name, population):
         to_add = {}
         for target_state, amount, _ in population:
-            to_add[(self.get_model_value(target_state),)] = amount
+            key = (self.get_model_value(target_state),)
+            if key not in to_add and amount > 0:
+                to_add[key] = amount
+            else:
+                to_add[key] += amount
         self.add_population({process_name: to_add})
 
 
@@ -254,6 +257,7 @@ class CompartProcessManager(AbstractProcessManager):
         # add populations to appropriate compartments for each process
         added = {}
         for process, spec in population_spec.items():
+            # print(process, spec)
             if process not in self.no_compart:
                 # retrieve the group manager associated to the process
                 manager = self[process]
@@ -277,6 +281,7 @@ class CompartProcessManager(AbstractProcessManager):
                     added[process] += qty
         if init:
             manager.update_counts()
+        # print(list(added.values()))
         nb = set(added.values())
         assert(len(nb) <= 1)
         if nb:
@@ -325,10 +330,14 @@ class CompartProcessManager(AbstractProcessManager):
         total = None
         if amount is not None:
             for name, proc in self._content.items():
-                keys, probas = zip(*[(key, comp.population/self.population)
-                                     for key, comp in proc.items()])
-                amounts = np.random.multinomial(amount, probas)
-                to_remove[name] = dict(zip(keys, amounts))
+                if name not in self.no_compart:
+                    keys, probs = zip(*[(key, comp.population)
+                                        for key, comp in proc._content.items()])
+                    s = sum(probs)
+                    probas = [p / s for p in probs]
+                    amounts = np.random.multinomial(amount, probas)
+                    to_remove[name] = dict(zip(keys, amounts))
+                    # print(to_remove)
             self.remove_population(to_remove)
             return to_remove
 
@@ -386,8 +395,13 @@ class CompartProcessManager(AbstractProcessManager):
             })
             if self.level in self.model.outputs and\
                'extra_vars' in self.model.outputs[self.level]:
-                res.update({name: self.get_model_value(name)\
-                            if name in self.model.parameters\
-                            else self.get_information(name)
-                            for name in self.model.outputs[self.level]['extra_vars']})
+                for name in self.model.outputs[self.level]['extra_vars']:
+                    if name in self.model.parameters:
+                        res[name] = self.get_model_value(name)
+                    else:
+                        try:
+                            value = self.get_information(name)
+                        except StateVarNotFoundException:
+                            value = np.nan
+                        res[name] = value
         return pd.DataFrame(res, index=[0])
