@@ -12,31 +12,30 @@
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -112,10 +111,25 @@ class MultiProcessManager(AbstractProcessManager):
         """
         if self.level in self.model.initial_conditions:
             conds = self.model.initial_conditions[self.level]
-            for proto, qty in conds['prototypes']:
-                to_add = [ self.new_atom(prototype=proto, execute_actions=True)
-                           for _ in range(int(self.get_model_value(qty)))
-                ]
+            for protos, qty, probas in conds['prototypes']:
+                proba_values = [self.get_model_value(p) for p in probas]
+                total_value = sum(proba_values)
+                assert(0 <= total_value <= 1)
+                ## if sum of probas < 1 add complement (even if
+                ## proba_values same size as protos) to ensure that
+                ## e.g. [0.1, 0.1] with amount 100 gives in average
+                ## [10, 10] individuals
+                if total_value < 1:
+                    proba_values += [1 - total_value]
+                amount = int(self.get_model_value(qty))
+                qty_by_proto = np.random.multinomial(amount, proba_values)
+                # truncate qty_by_proto if size > nb of protos actually used
+                qty_by_proto = qty_by_proto[:len(protos)]
+                to_add = []
+                for proto, nb in zip(protos, qty_by_proto):
+                    to_add += [ self.new_atom(prototype=proto, execute_actions=True)
+                                for _ in range(nb)
+                    ]
                 self.add_atoms(to_add, init=True)
 
     def add_host(self, host):
@@ -159,11 +173,32 @@ class MultiProcessManager(AbstractProcessManager):
                                     compart_class=compart_class)
 
 
-    def select_atoms(self, variable, state=None, value=None, process=None):
-        """Return a list of atoms having `variable` equal to the specified
-        `state_name`.
+    def select_atoms(self, variable=None, state=None, value=None, process=None):
+        """Return a list of atoms selected by specific *value* or *state* of a
+        *variable*.
 
+        Parameters
+        ----------
+        variable: str
+            the variable to be compared to the *value* - if ``None``,
+            all atoms are selected.
+        state: str
+            a name of the state used for the selection (instead of *value*)
+        value: object
+            the value to use for selection - if ``None``, the value is
+            replaced with the model state specified in parameter
+            *state*
+        process: str
+            name of a process associated to a specific grouping based
+            on the *variable* (to accelerate search)
+
+        Returns
+        -------
+        list:
+            the list of matching agents in the sublevel
         """
+        if variable is None:
+            return list(self['MASTER'])
         if state is not None:
             value = self.model.get_value(state)
         if process is not None:
@@ -224,13 +259,13 @@ class MultiProcessManager(AbstractProcessManager):
         if sublevel is None:
             sublevel = self.get_default_sublevel()
         atom_class = self.get_agent_class_for_sublevel(sublevel)
-        args.update(model=self.model, level=sublevel, step=self.statevars.step)
+        args.update(model=self.model, simu_id=self.statevars.simu_id,
+                    level=sublevel, step=self.statevars.step)
         if prototype is not None:
             args.update(prototype=prototype, execute_actions=execute_actions)
         elif custom_prototype is not None:
             args.update(custom_prototype=custom_prototype, execute_actions=execute_actions)
         new_atom = atom_class(**args)
-        # new_atom.statevars.step = self.statevars.step
         # initialize MASTER host
         new_atom.add_host(self['MASTER'])
         add_all_test_properties(new_atom)
