@@ -12,31 +12,30 @@
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -63,7 +62,7 @@ from   copy                    import deepcopy
 import yaml
 import dateutil.parser         as     dup
 
-from   sympy                   import sympify, pretty, Symbol
+from   sympy                   import sympify, Symbol
 
 from   sortedcontainers        import SortedSet, SortedDict
 from   jinja2                  import Environment, PackageLoader
@@ -71,7 +70,7 @@ from   jinja2                  import Environment, PackageLoader
 import networkx                as     nx
 
 from   emulsion.tools.state    import StateVarDict
-from   emulsion.tools.misc     import read_from_file, load_class, create_new_serial
+from   emulsion.tools.misc     import read_from_file, load_class, create_new_serial, create_weighted_random
 from   emulsion.tools.calendar import EventCalendar
 
 from   emulsion.model.functions      import DEFAULT_LEVEL_INFO, DEFAULT_GROUPING_INFO,\
@@ -357,7 +356,7 @@ class EmulsionModel(object):
         origin: 'May 1'
         calendar:
           name:
-          period: 52
+          period: {days: 52}
           events:
             spring: {begin: 'April 8', end: 'April 23'}
             summer: {begin: 'July 8', end: 'September 3'}
@@ -394,26 +393,49 @@ class EmulsionModel(object):
         for cal_name, cal in calendar_desc.items():
             events = {}
             if 'events' in cal:
+                cal_period = timedelta(**cal['period']) if 'period' in cal else None
                 for event_name, definition in cal['events'].items():
+                    # register the event name in the events namespace
                     self._event_namespace[event_name] = Symbol(event_name)
+                    # if keyword "date" is used, begin = end
                     if 'date' in definition:
                         events[event_name] = (dup.parse(definition['date']),
                                               dup.parse(definition['date']))
                     else:
                         events[event_name] = (dup.parse(definition['begin']),
                                               dup.parse(definition['end']))
+                        # if begin and end date are specified,
+                        # register events corresponding to "begin_event"
+                        # and "end_event"
                         for keyword in ['begin', 'end']:
                             event_limit = keyword + '_' + event_name
                             self._event_namespace[event_limit] = Symbol(event_limit)
                             events[event_limit] = (dup.parse(definition[keyword]),
                                                    dup.parse(definition[keyword]))
-            # cal_name = cal['name']
+                    # register the durations of each event
+                    # compute duration of event
+                    begin, end = events[event_name]
+                    if end >= begin:
+                        dur = end - begin
+                    else:
+                        dur = end - begin + cal_period
+                    # compute duration of 1 time unit
+                    unit = {self.time_unit: 1}
+                    dur_unit = timedelta(**unit)
+                    if (dur_unit > timedelta(days=1)):
+                        dur += timedelta(days=1)
+                    else: # 1h / 1min / 1s ...
+                        dur += dur_unit
+                    # convert duration to time units
+                    event_duration = dur / dur_unit
+                    duration_param = 'duration_of_{}'.format(event_name)
+                    self._namespace[duration_param] = Symbol(duration_param)
+                    self._values[duration_param] = sympify(event_duration)
+                    # cal_name = cal['name']
             self.calendars[cal_name] = EventCalendar(cal_name,
                                                      self.step_duration,
                                                      self.origin_date,
-                                                     period=timedelta(**cal['period'])\
-                                                       if 'period' in cal\
-                                                       else None,
+                                                     cal_period,
                                                      **events)
             for event in self.calendars[cal_name].get_events():
                 self._events[event] = cal_name
@@ -443,7 +465,7 @@ class EmulsionModel(object):
 
         """
         if 'modules' in self._description:
-            self.modules = self._description['modules']
+            self.modules += self._description['modules']
         if 'parameters' in self._description:
             for (key, val) in self._description['parameters'].items():
                 self.parameters[key] = sympify(val['value'],
@@ -539,8 +561,21 @@ class EmulsionModel(object):
                                 )
                     else:
                         ## build initial conditions for individual-based model
+                        protos = list_items['prototype']
+                        if not isinstance(protos, list):
+                            protos = [protos]
+                            list_items['prototype'] = protos
+                        amount = list_items['amount']
+                        if 'proba' in list_items:
+                            probas = list_items['proba']
+                            if not isinstance(probas, list):
+                                probas = [probas]
+                        else:
+                            probas = ['1'] * len(protos)
+                        list_items['proba'] = probas
+                        assert(0 <= len(protos) - len(probas) <= 1)
                         self.initial_conditions[level]['prototypes'].append(
-                            (list_items['prototype'], self.add_expression(list_items['amount'])))
+                            (protos, self.add_expression(amount), [self.add_expression(p) for p in probas]))
         # print(self._description['initial_conditions'])
         # print(self.initial_conditions)
 
@@ -568,11 +603,26 @@ class EmulsionModel(object):
                     for (key, val) in list_item.items():
                         self.prototypes[level][key] = {}
                         for variable, value in val.items():
+                            result = value
                             if variable != 'desc':
                                 if value == 'random':
                                     result = '_random_' + variable
                                 elif value == 'default':
                                     result = '_default_' + variable
+                                elif value == 'next_state':
+                                    result = '_next_state_' + variable
+                                    # associate to a function that returns the successor of the state contained in the variable (or a random state if the variable was not defined)
+                                    self._values[result] = lambda agent: agent.statevars[variable].successor if variable in agent.statevars else agent.model.state_machines[variable].get_random_state()
+                                elif value == 'previous_state':
+                                    result = '_previous_state_' + variable
+                                    # associate to a function that returns the predecessor of the state contained in the variable (or a random state if the variable was not defined)
+                                    self._values[result] = lambda agent: agent.statevars[variable].predecessor if variable in agent.statevars else agent.model.state_machines[variable].get_random_state()
+                                elif type(value) == str and value.startswith('random('):
+                                    # result = '_weighted_random_' + variable
+                                    result = '{}_{}'.format(variable, value)
+                                    if value[6] == '(' and value[-1] == ')':
+                                        weights = [self.add_expression(e) for e in value[7:-1].split(',')]
+                                    self._values[result] = create_weighted_random(variable, weights, model=self)
                                 elif type(value) == str and value.startswith('serial'):
                                     result = '_serial_' + variable
                                     if value != 'serial' and value[6] == '(' and value[-1] == ')':
@@ -580,8 +630,8 @@ class EmulsionModel(object):
                                     else:
                                         start, end = 0, None
                                     self._values[result] = create_new_serial(end=end, model=self)
-                                else:
-                                    result = value
+                                elif value in self.statevars:
+                                    self._values[result] = lambda agent: agent.statevars[result]
                                 self.prototypes[level][key][variable] = result
 
 
@@ -689,7 +739,8 @@ class EmulsionModel(object):
                     if 'sublevels' in default:
                         if 'contains' in desc:
                             for sublevel in desc['contains']:
-                                if (not self.levels[sublevel]['module'].startswith('emulsion.agent'))\
+                                if ('module' not in self.levels[sublevel] or\
+                                    (not self.levels[sublevel]['module'].startswith('emulsion.agent')))\
                                    and ('super' not in self.levels[sublevel]):
                                     self.levels[sublevel]['super'] = default['sublevels']
                 if 'aggregate_vars' in desc:
@@ -903,9 +954,10 @@ class EmulsionModel(object):
         name.
 
         """
-        pretty_name = pretty(sympify(name, locals=self._namespace))
-        param_name = name if pretty_name == name\
-                     else "{} ({})".format(pretty_name, name)
+        # pretty_name = pretty(sympify(name, locals=self._namespace))
+        # param_name = name if pretty_name == name\
+        #              else "{} ({})".format(pretty_name, name)
+        param_name = name
         return "{} [parameter]:\n\t{: <72}\n\t{}".format(
             param_name,
             self._description['parameters'][name]['desc'],
@@ -917,9 +969,10 @@ class EmulsionModel(object):
         name.
 
         """
-        pretty_name = pretty(sympify(name, locals=self._namespace))
-        var_name = name if pretty_name == name\
-                   else "{} ({})".format(pretty_name, name)
+        # pretty_name = pretty(sympify(name, locals=self._namespace))
+        # var_name = name if pretty_name == name\
+        #            else "{} ({})".format(pretty_name, name)
+        var_name = name
         return "{} [variable]:\n\t{: <72}".format(
             var_name,
             self._description['statevars'][name]['desc'],
