@@ -12,31 +12,30 @@
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -59,7 +58,7 @@
 from   functools               import partial
 
 import numpy                   as     np
-from   sympy                   import sympify, pretty
+from   sympy                   import sympify
 
 from   sortedcontainers        import SortedSet
 
@@ -68,7 +67,7 @@ from   emulsion.agent.action   import AbstractAction
 from   emulsion.tools.state    import StateVarDict, EmulsionEnum
 
 from   emulsion.model.functions import ACTION_SYMBOL, WHEN_SYMBOL, ESCAPE_SYMBOL,\
-    COND_SYMBOL, CROSS_SYMBOL, EDGE_KEYWORDS,\
+    COND_SYMBOL, CROSS_SYMBOL, EDGE_KEYWORDS, CLOCK_SYMBOL,\
     make_when_condition, make_duration_condition, make_duration_init_action
 
 from   emulsion.model.exceptions     import SemanticException
@@ -133,25 +132,11 @@ class StateMachine(object):
         return self.stateprops[state_name][property_name]
 
     def build_states(self):
-        """Parse the description of the state machine and extract the
-        existing states. States are described as list items, endowed
-        with key-value properties. It is recommended to define only
-        one state per list item (especially to ensure that states are
-        always stored in the same order in all executions).
-
-        Example of YAML specification:
-        ------------------------------
-        states:
-          - S:
-              name: Susceptible
-              desc: 'non-shedder cows without antibodies'
-          - I+:
-              name: Infectious plus
-              desc: 'shedder cows with antibodies'
-              fillcolor: orange
-              on_stay:
-                - increase: total_E
-                  rate: Q1
+        """Parse the description of the state machine and extract the existing
+        states. States are described as list items, endowed with
+        key-value properties. Only one state per list item is allowed
+        (to ensure that states are always stored in the same order in
+        all executions).
 
         """
         states = []
@@ -178,6 +163,18 @@ class StateMachine(object):
                     default_state = name
                 self.stateprops[name] = {k: self.model.add_expression(v)
                                          for k, v in value['properties'].items()}
+                # store special properties: "next: aState" and
+                # "previous: aState" which define
+                # successor/predecessor when using "next_state" and
+                # "previous_state" in prototype definition (otherwise,
+                # next and previous are defined as non-autoremove
+                # states after and before in the list)
+                if 'next' in value:
+                    value['properties']['next'] = value['next']
+                    self.stateprops[name]['next'] = value['next']
+                if 'previous' in value:
+                    value['properties']['previous'] = value['previous']
+                    self.stateprops[name]['previous'] = value['previous']
                 # store other information
                 self._statedesc[name] = value
                 # and retrieve available actions if any
@@ -187,34 +184,71 @@ class StateMachine(object):
         # build the enumeration of the states
         self.states = EmulsionEnum(self.machine_name.capitalize(),
                                    states, module=__name__)
+        # link the states to their state machine
         self.states.state_machine = self
+        # define the default value for "autoremove" at the enumeration level
         self.states.autoremove = False
 
         for state in self.states:
+            # check that state names are unique
             if state.name in self.model.states:
                 other_machine = self.model.states[state.name].__class__.__name__
                 raise SemanticException(
                     'Conflict: State %s found in statemachines %s and %s' %
                     (state.name, other_machine, state.__class__.__name__))
+            # check that state names are not parameters
             if state.name in self.model.parameters:
                 raise SemanticException(
                     'Conflict: State %s of statemachines %s found in parameters'
                     % (state.name, state.__class__.__name__))
+            # associate the state with the state name in the model
             self.model.states[state.name] = state
+            # update the autoremove value if the state is defined as such
             if self.stateprops[state.name]['autoremove']:
                 state.autoremove = True
+            # set next and previous to the state itself (to ensure
+            # that all states have a predecessor/successor)
+            state.predecessor = state
+            state.successor = state
 
+        # handle default states
         self.states.is_default = False
+        # if a default state is defined, store it and limit the list
+        # of "available" states
         if default_state is not None:
             self.states[default_state].is_default = True
             self.states.default = self.states[default_state]
             self.states.available = (self.states.default,)
         else:
+        # otherwise the list of available states is simply those which
+        # are not autoremove
             self.states.default = None
             self.states.available = tuple(s for s in self.states if not s.autoremove)
         # print(self.states, self.states.default, self.states.available)
 
+        # define "successor" and "predecessor" properties
+        # start from the list of non-autoremove states
+        usable = [s for s in self.states if not s.autoremove]
+        for state in usable:
+            # if the state has a "predecessor" state, use it
+            if 'previous' in self.stateprops[state.name]:
+                state.predecessor = self.states[self.stateprops[state.name]['previous']]
+            else:
+                # otherwise use the previous state in the list, the
+                # first one being its own previous state
+                state.predecessor = usable[max(0, usable.index(state) - 1)]
+            # if the state has a "successor" state, use it
+            if 'next' in self.stateprops[state.name]:
+                state.successor = self.states[self.stateprops[state.name]['next']]
+            else:
+                # otherwise use the next state in the list, the
+                # last one being its own next state
+                state.successor = usable[min(len(usable)-1, usable.index(state) + 1)]
+
+        # define function used from model to provide a random state
         self.model._values['_random_' + self.machine_name] = self.get_random_state
+        # define function used from model to provide the default state
+        # if any, otherwise a random state among non-autoremove ones
         if self.states.default:
             self.model._values['_default_' + self.machine_name] = self.get_default_state
         else:
@@ -226,7 +260,7 @@ class StateMachine(object):
         return np.random.choice([state for state in self.states if not state.autoremove])
 
     def get_default_state(self, caller=None):
-        """Return a random state for this state machine."""
+        """Return the default state for this state machine."""
         return self.states.default
 
     @property
@@ -278,7 +312,8 @@ class StateMachine(object):
                       if k != 'from' and k != 'to'}
             for kwd in EDGE_KEYWORDS:
                 if kwd in others:
-                    parm = pretty(sympify(others[kwd], locals=self.model._namespace))
+                    # parm = pretty(sympify(others[kwd], locals=self.model._namespace))
+                    parm = others[kwd]
                     label = '{}: {}'.format(kwd, parm)
             # label = ', '.join([pretty(sympify(x, locals=self.model._namespace))
             #                    for x in others.values()])
@@ -524,7 +559,7 @@ class StateMachine(object):
                             )
                         )
                         understood = True
-                for keyword in ['clone', 'produce_offspring']:
+                for keyword in ['become', 'clone', 'produce_offspring']:
                     if keyword in d_action:
                         amount = d_action['amount'] if 'amount' in d_action else None
                         probas = d_action['proba'] if 'proba' in d_action else None
@@ -538,7 +573,19 @@ class StateMachine(object):
                             )
                         )
                         understood = True
-                for keyword in ['become', 'message']:
+                for keyword in ['log_vars']:
+                    if keyword in d_action:
+                        vars = [self.model.add_expression(varname)
+                                for varname in d_action[keyword]]
+                        l_actions.append(
+                            AbstractAction.build_action(
+                                keyword,
+                                parameter=None,
+                                l_params=vars
+                            )
+                        )
+                        understood = True
+                for keyword in ['message', 'record_change']:
                     if keyword in d_action:
                         l_actions.append(
                             AbstractAction.build_action(
@@ -574,19 +621,20 @@ class StateMachine(object):
 
         rankdir = "LR" if self.graph.edges() else "TB"
         output = '''digraph {
+          charset="utf-8"
         '''
         output += '''\trankdir={};
         '''.format(rankdir)
         output += '''
         \tnode[fontsize=16, fontname=Arial, shape=box, style="filled,rounded"];
-        \tedge[minlen=1.5, penwidth=1.5, tailtooltip="", headtooltip=""];
+        \tedge[minlen=1.5, fontname=Times, penwidth=1.5, tailtooltip="", headtooltip=""];
 
         '''
         for state in self.states:
             name = state.name
             name_lab = name
             if 'duration' in self._statedesc[name]:
-                name_lab += '&nbsp;&#9719;'
+                name_lab += '&nbsp;{}'.format(CLOCK_SYMBOL)
             actions = 'shape="Mrecord", label="{}", '.format(name_lab)
             nodestyle = "filled,rounded"
             if state.is_default:
@@ -637,5 +685,5 @@ class StateMachine(object):
                                from_, to_, desc['label'], desc['labeltooltip'],
                                tail, head, edgetip, desc['type_id'].linestyle)
         output += '}'
-        with open(filename, 'w') as f:
+        with open(filename, 'w', encoding="utf8") as f:
             f.write(output)
