@@ -10,31 +10,30 @@ Classes and functions for the definition of Emulsion calendars.
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -73,6 +72,7 @@ class InvalidIntervalException(Exception):
 
     """
     def __init__(self, begin, end):
+        """Create the exception with the incorrect *begin* and *end* dates."""
         super().__init__()
         self.value = (begin, end)
 
@@ -93,20 +93,26 @@ def date_in(begin, end, period=None):
     test relies upon the periodicity, otherwise dates are considered
     absolute.
 
-    Args:
-        begin:  a ``datetime.datetime`` object
-        end:    a ``datetime.datetime`` object
-        period: a ``datetime.timedelta`` object
+    Parameters
+    ----------
+    begin: ``datetime.datetime``
+        the date when the interval begins
+    end: ``datetime.datetime``
+        the date when the interval ends
+    period: ``datetime.timedelta``
+        the duration of the period after which the calendar cycles
 
-    Return:
-        A function which maps a date (``datetime.datetime``) to a
+    Returns
+    -------
+    lambda:
+        a function which maps a date (``datetime.datetime``) to a
         ``bool`` to indicate whether or not the date belongs to the
         *begin*-*end* interval.
 
-    Raises:
-
-        InvalidIntervalexception: if *begin* is posterior to *end* in
-          a non-periodic calendar.
+    Raises
+    ------
+    InvalidIntervalexception
+        if *begin* is posterior to *end* in a non-periodic calendar
     """
     if period is not None:
         begin, end = begin % period, end % period
@@ -124,7 +130,7 @@ def date_in(begin, end, period=None):
                 date appears before the begin date in a civil year.
 
                 """
-                return (date % period) <= begin or (date % period) >= end
+                return (date % period) <= end or (date % period) >= begin
     else:
         if begin <= end:
             def test_date(date):
@@ -157,17 +163,21 @@ class EventCalendar:
                  period=None, **initial_dates):
         """Initialize the calendar using the specified information.
 
-        Args:
-        - calendar_name: name of the calendar
-        - step_duration: actual duration of one simulation step
-          (timedelta)
-        - origin: date/time value of the beginning of the calendar
-          (datetime)
-        - period: if the calendar is periodic, actual duration of the
-          period (timedelta), None otherwise
-        - initial_dates: initial dictionary of events (event name as
-          key, dict as value with either begin/end dates or one
-          date)
+        Parameters
+        ----------
+        calendar_name: string
+            the name of the calendar
+        step_duration: ``datetime.timedelta``
+            actual duration of one simulation step
+        origin: ``datetime.datetime``
+            date/time value of the beginning of the calendar
+        period: ``datetime.timedelta``
+            if the calendar is periodic, actual duration of the
+            period, None otherwise
+        initial_dates: dict
+             initial dictionary of events (event name as key, dict as
+             value with either begin/end dates or one single date if
+             punctual event)
 
         """
         self.calendar_name = calendar_name
@@ -187,34 +197,77 @@ class EventCalendar:
         """Advance the current date by the specified number of
         simulation steps.
 
+        Parameters
+        ----------
+        steps: int
+            number of steps to 'add' to the current date
         """
         self.current_date += self.step_duration * steps
 
     def step_to_date(self, step):
-        """Return the date when the specified step begins."""
+        """Return the date when the specified step begins.
+
+        Parameters
+        ----------
+        step: int
+            the time step to convert to a date
+
+        Returns
+        -------
+        d: ``datetime.datetime``
+            the corresponding date
+        """
         return self.origin + self.step_duration * step
 
     def date_to_step(self, date):
-        """Return the step including the specified date."""
+        """Return the step corresponding the specified date.
+
+        Parameters
+        ----------
+        date: ``datetime.datetime``
+            the date to convert into time steps
+
+        Returns
+        -------
+        t: int
+            the time step during which the specified date occurs (the
+            date occurs between the returned time step *t* and time
+            step *t+1*)
+
+        """
         return (date - self.origin) // self.step_duration
 
 
-    def add_event(self, name, value):
-        """Add the specified event to the calendar. An event is
-        characterized by its name and a tuple indicating the begin and
+    def add_event(self, name, begin_end):
+        """Add the specified event to the calendar. An event is characterized
+        by its *name* and a *begin_end* tuple indicating the begin and
         end dates.
 
+        Parameters
+        ----------
+        name: str
+            the name of the event
+        begin_end: tuple
+           a tuple composed of the begin date and the end date
+           (possibly identical for handling punctual events)
+
         """
-        begin_date, end_date = value
+        begin_date, end_date = begin_end
         begin = self.date_to_step(begin_date)
         end = self.date_to_step(end_date)
         if end < begin and self.period is None:
             raise InvalidIntervalException(begin, end)
-        self.description[name] = value
+        self.description[name] = begin_end
         self.events[name] = date_in(begin, end, self.period)
 
     def get_events(self):
-        """Return the list of events contained in the current calendar."""
+        """Return the list of events contained in the current calendar.
+
+        Returns
+        -------
+        l: list
+            the list of all event names
+        """
         return list(self.description.keys())
 
     def __getitem__(self, name):
