@@ -1,4 +1,6 @@
-"""Collection of various useful functions used in EMULSION framework,
+""".. module:: emulsion.tools.misc
+
+Collection of various useful functions used in EMULSION framework,
 especially regarding introspection.
 
 """
@@ -10,31 +12,30 @@ especially regarding introspection.
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -83,16 +84,23 @@ def load_class(module=None, class_name: str = None, options: dict = {}):
     """Dynamically load the class with the specified *class_name* from the
     given *module*.
 
-    Args:
-        module: a Python module, where the class is expected to be located.
-        class_name: the name of the class to load
-        options: some options
+    Parameters
+    ----------
+    module:
+        a Python module, where the class is expected to be located.
+    class_name: str
+        the name of the class to load
+    options:
+        some options
 
-    Returns:
-        A tuple composed of the class (type) and the options.
+    Returns
+    -------
+    tuple:
+        a tuple composed of the class (type) and the options.
 
-    Todo:
-        - clarify the role of *options*
+    Todo
+    ----
+    - clarify the role of *options*
     """
     mod = import_module(module)
     return getattr(mod, class_name), options
@@ -101,11 +109,15 @@ def load_module(module_name: str):
     """Dynamically load the module with the specified *module_name* and
     return it.
 
-    Args:
-        module_name: the name of a valid Python module (accessible in the
-            ``PYTHONPATH`` environment variable).
+    Parameters
+    ----------
+    module_name: str
+        the name of a valid Python module (accessible in the
+        ``PYTHONPATH`` environment variable).
 
-    Returns:
+    Returns
+    -------
+    ref:
         A reference to the Python module.
 
     """
@@ -128,17 +140,24 @@ def rates_to_probabilities(total_rate: float, rate_values: List[float],
     .. math::
         p_i = \\frac{\\rho_i}{\\sum_i \\rho_i} (1 - p_0)
 
-    Args:
-        total_rate: the total exit rate, used for normalization
-            purposes. If *rate_values* represent all possible exit
-            rates, *total_rate* is their sum.
-        rate_values: the list of rates to transform
-        delta_t: the value of the time step (expressed in time units)
+    Parameters
+    ----------
+    total_rate: float
+        the total exit rate, used for normalization purposes. If
+        *rate_values* represent all possible exit rates, *total_rate*
+        is their sum.
+    rate_values: list
+        the list of rates to transform
+    delta_t: float
+        the value of the time step (expressed in time units)
 
-    Returns:
-        The list of probabilities corresponding to the *rate_values*.
+    Returns
+    -------
+    list:
+        the list of probabilities corresponding to the *rate_values*.
 
-    See Also:
+    See Also
+    --------
         `emulsion.tools.misc.probabilities_to_rates`_
 
     """
@@ -160,12 +179,17 @@ def aggregate_probability(probability: float, delta_t: float) -> float:
     a probability for events tested each time unit, into the
     probability for the specified time step (*delta_t*).
 
-    Args:
-        probability: the probability value of an event (during 1 time unit)
-        delta_t: the value of the time step (expressed in time units)
+    Parameters
+    ----------
+    probability: float
+        the probability value of an event (during 1 time unit)
+    delta_t: float
+        the value of the time step (expressed in time units)
 
-    Returns:
-        The probability of the event during *delta_t* time units.
+    Returns
+    -------
+    float:
+        the probability of the event during *delta_t* time units.
     """
     return 1 - (1 - probability)**delta_t
 
@@ -179,13 +203,17 @@ def aggregate_probabilities(probability_values: Iterable[float],
     probabilities of events duting one time unit, compute the
     probabilities for the specified time step (*delta_t*).
 
-    Args:
-        probability_values: a list of probability values for several events
-            (during 1 time unit)
-        delta_t: the value of the time step (expressed in time units)
+    Parameters
+    ----------
+    probability_values: list
+        a list of probability values for several events (during 1 time unit)
+    delta_t: float
+        the value of the time step (expressed in time units)
 
-    Returns:
-        The probabilities of the same events during *delta_t* time units.
+    Returns
+    -------
+    list:
+        the probabilities of the same events during *delta_t* time units.
     """
     return probability_values if delta_t == 1\
         else tuple(aggregate_probability(p, delta_t)
@@ -200,15 +228,21 @@ def probabilities_to_rates(probability_values: Iterable[float]) -> List[float]:
     last value is expected to represent the probability of staying in
     the current state.
 
-    Args:
-        probability_values: a list of probabilities, the last one representing
-            the probability to stay in the current state
+    Parameters
+    ----------
+    probability_values: list
+        a list of probabilities, the last one representing the
+        probability to stay in the current state
 
-    Returns:
-        A list of rates corresponding to those probabilities.
+    Returns
+    -------
+    list:
+        a list of rates corresponding to those probabilities.
 
-    See Also:
+    See Also
+    --------
         `emulsion.tools.misc.rates_to_probabilities`_
+
     """
     if probability_values[-1] == 1:
         return [0] * (len(probability_values) - 1)
@@ -232,13 +266,17 @@ def rewrite_keys(name, position, change_list):
 def count_population(agents_or_pop: Tuple) -> Union[int, float]:
     """Return the amount of atoms represented in *agents_or_pop*.
 
-    Args:
-        agents_or_pop: a tuple: either ('population', qty) or
-            ('agents', list of agents).
+    Parameters
+    ----------
+    agents_or_pop: tuple
+      either ('population', qty) or ('agents', list of agents)
 
-    Returns:
-        The amount corresponding to the population: generally, an int
+    Returns
+    -------
+    int or float:
+        the amount corresponding to the population: generally, an int
         value, but deterministic models produce float values.
+
     """
     return agents_or_pop[POPULATION]\
       if 'population' in agents_or_pop\
@@ -252,13 +290,19 @@ def select_random(origin: Iterable, quantity: int,
     *origin* population proves too small, all available agents are
     taken, irrespective to the *quantity*.
 
-    Args:
-        origin: the population where agents must be selected
-        quantity: the number of agents to select in the population
-        exclude: agents which are not available for the selection
+    Parameters
+    ----------
+    origin: iterable
+        the population where agents must be selected
+    quantity: int
+        the number of agents to select in the population
+    exclude: set
+        agents which are not available for the selection
 
-    Returns:
-        A list of randomly selected agents according to the above constraints.
+    Returns
+    -------
+    list:
+        a list of randomly selected agents according to the above constraints.
     """
     content = [unit for unit in origin if unit not in exclude]
     size = len(content)
@@ -273,17 +317,21 @@ def read_from_file(filename: str):
     """Read the specified YAML *filename* and return the corresponding
     Python document.
 
-    Args:
-        filename: the name of the YAML file to load
+    Parameters
+    ----------
+    filename: str
+        the name of the YAML file to load
 
-    Returns:
-        A Python object built by parsing of the YAML file, i.e. either
+    Returns
+    -------
+    object:
+        a Python object built by parsing of the YAML file, i.e. either
         a dict, list, or even str/int/etc... (Most YAML document will
         produce dictionaries.)
 
     """
     with open(filename, 'r') as fil:
-        description = yaml.load(fil)
+        description = yaml.safe_load(fil)
     return description
 
 
@@ -292,13 +340,19 @@ def retrieve_value(value_or_function, agent):
     *value_or_function* if it is a true value, or computed from this
     parameter seen as a function, with the specified *agent* as argument.
 
-    Args:
-        value_or_function: either a callable (function that applies to
-            an agent to retrieve an individual value), or the value itself
-        agent: the agent to use as parameter of the callable if necessary
+    Parameters
+    ----------
+    value_or_function:
+        either a callable (function that applies to an agent to
+        retrieve an individual value), or the value itself
+    agent:
+        the agent to use as parameter of the callable if necessary
+
+    Returns
+    -------
+    value:
+        the expected value (agent-based or not).
 
-    Returns:
-        The expected value (agent-based or not).
     """
 
     return value_or_function(agent)\
@@ -313,13 +367,19 @@ def moving_average(values, window_size, mode='same'):
     boundary effects, use ``mode='valid'``, which produce a result of
     size ``len(values) - window_size + 1``.
 
-    Args:
-        values: array-like, contains the values for moving average computation
-        window_size: width of the moving average window
-        mode: a parameter for `numpy.convolve`
+    Parameters
+    ----------
+    values: array-like
+        contains the values for moving average computation
+    window_size: int
+        width of the moving average window
+    mode: str
+        a parameter for `numpy.convolve`
 
-    Returns:
-        A numpy ``nd_array`` containing the values of the moving average.
+    Returns
+    -------
+    nd_array:
+        a numpy ``nd_array`` containing the values of the moving average.
 
     """
     window = np.ones(int(window_size)) / float(window_size)
@@ -329,7 +389,6 @@ def moving_average(values, window_size, mode='same'):
 moving_average.__USER_FUNCTION__ = ['Computations']
 
 
-
 def add_new_property(agent, property_name, getter_function):
     """Add a new property to an agent.
 
@@ -339,11 +398,14 @@ def add_new_property(agent, property_name, getter_function):
     since agents must add the name of the property to their
     ``_mbr_cache`` attribute.
 
-    Args:
-        agent (AbstractAgent): the agent to which the property must be
-            added
-        property_name: the name of the property
-        getter_function: the function upon which the property is built
+    Parameters
+    ----------
+    agent: AbstractAgent
+        the agent to which the property must be added
+    property_name: str
+        the name of the property
+    getter_function: lambda
+        the function upon which the property is built
 
     """
     setattr(agent.__class__, property_name, property(getter_function))
@@ -412,11 +474,15 @@ def find_operator(operator: str):
     is interpreted as the partial function
     ``numpy.percentile(q=int(XX))``.
 
-    Args:
-        operator: the name of the aggregation operator
+    Parameters
+    ----------
+    operator: str
+        the name of the aggregation operator
 
-    Returns:
-        A function that takes a list (or array-like) as input and
+    Returns
+    -------
+    lambda:
+        a function that takes a list (or array-like) as input and
         returns the application of the *operator* to the values.
 
     """
@@ -440,11 +506,16 @@ def create_aggregator(sourcevar: str, operator: str):
     agents contained in a given host, and reduce them to one avalue
     using the specified *operator*.
 
-    Args:
-        sourcevar: the name of the variable to collect in the sublevel
-        operator: the name of the operator to apply to the collected values
+    Parameters
+    ----------
+    sourcevar: str
+        the name of the variable to collect in the sublevel
+    operator: str
+        the name of the operator to apply to the collected values
 
-    Returns:
+    Returns
+    -------
+    lambda:
         A function which can be applied to a `MultiProcessManager`
         agent (i.e. with explicit sublevel agents) which returns the
         aggregated values for the whole population.
@@ -463,15 +534,22 @@ def create_group_aggregator(sourcevar: str, operator: str, process_name: str,
     aggregating the values using *operator*. The functions can handle
     groupings with an arbitrary number of states.
 
-    Args:
-        sourcevar: the name of the variable to collect in the sublevel
-        operator: the name of the operator to apply to the collected values
-        process_name: the name of the process associated with the
-          grouping for which the population getter is created
-        group_name: either a string representing the state name (if
-          only one), or a tuple of several state names
+    Parameters
+    ----------
+    sourcevar: str
+        the name of the variable to collect in the sublevel
+    operator: str
+        the name of the operator to apply to the collected values
+    process_name: str
+        the name of the process associated with the grouping for which
+        the population getter is created
+    group_name: str or tuple
+        either a string representing the state name (if only one), or
+        a tuple of several state names
 
-    Returns:
+    Returns
+    -------
+    lambda:
         A function which can be applied to an `MultiProcessManager`
         agent (i.e. with explicit sublevel agents) which returns the
         aggregated value for the specified group.
@@ -498,14 +576,20 @@ def create_atoms_aggregator(sourcevar: str, operator: str,
     values using *operator*. This function is intended to work on
     `IBMProcessManager` agents, which do not benefit from groupings.
 
-    Args:
-        sourcevar: the name of the variable to collect in the sublevel
-        operator: the name of the operator to apply to the collected values
-        machine_name: the name of the state machine for which the
-          getter is created
-        state_name: the state name
+    Parameters
+    ----------
+    sourcevar: str
+        the name of the variable to collect in the sublevel
+    operator: str
+        the name of the operator to apply to the collected values
+    machine_name: str
+        the name of the state machine for which the getter is created
+    state_name: str
+        the state name
 
-    Returns:
+    Returns
+    -------
+    lambda:
         A function which can be applied to an `IBMProcessManager`
         agent (i.e. with explicit but ungrouped sublevel agents) which
         returns the aggregated value for the specified group.
@@ -538,3 +622,45 @@ def create_new_serial(end=None, model=None):
     """
     generator = serial(start=0, end=end, model=model)
     return lambda self: next(generator)
+
+
+def create_weighted_random(machine_name, weights, model=None):
+    """Create a random choice function which returns a random state from
+    the given state machine (among non-autoremove states), according
+    to the weights. Weights are interpreted either directly as
+    probabilities (if the number of weights is stricly one below the
+    number of available states, the last state getting the complement
+    to 1), or as true weights which are then normalized to be used as
+    probabilities.
+
+    Parameters:
+    -----------
+    machine_name: str
+        the name of the state machine where the states must be chosen
+        among the *N* non-autoremove states.
+    weights: list
+        a list of *N* or *N-1* model expressions assumed to produce positive numbers
+    model: EmulsionModel
+        the model where this function is defined
+
+    Returns
+    -------
+    lambda: a function that returns a random state according to the
+        values of the weights list, interpreted either as
+        probabilities (if size *N-1*) or as weights (if size *N*)
+        which are then normalized to provide probabilities
+
+    """
+    machine = model.state_machines[machine_name]
+    states = [state for state in machine.states if not state.autoremove]
+    assert(len(states) - len(weights) <= 1)
+    def weighted_random(agent):
+        values = [agent.get_model_value(weight) for weight in weights]
+        total = sum(values)
+        if len(values) < len(states):
+            assert(0 <= total <= 1)
+            probas = values + [1 - total]
+        else:
+            probas = [v / total for v in values]
+        return np.random.choice(states, p=probas)
+    return weighted_random
