@@ -11,31 +11,30 @@ Plotting tools... to be improved!
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -60,7 +59,9 @@ import sys
 import pandas                       as pd
 import numpy                        as np
 
-from   scipy.misc                   import imread
+# from   scipy.misc                   import imread
+from   PIL                          import Image
+from   xml.etree                    import ElementTree
 from   bokeh.plotting               import figure, output_file, show
 from   bokeh.models                 import Legend, ColumnDataSource, Range1d
 from   bokeh.models.tools           import HoverTool, CrosshairTool
@@ -167,8 +168,17 @@ def build_machine_plot(machine_name, model, params):
     machine_url = model.model_name + '_' + machine_name + '_machine.' + params.img_format
     machine_path = Path(params.figure_dir, machine_url)
     try:
-        img = imread(machine_path)
-        height, width = img.shape[0:2]
+        # img = imread(machine_path)
+        # height, width = img.shape[0:2]
+        if params.img_format == 'svg':
+            img = ElementTree.parse(machine_path)
+            root = img.getroot()
+            # retrieve dimensions (in pts)
+            width = int(''.join(i for i in root.attrib['width'] if i.isdigit()))
+            height = int(''.join(i for i in root.attrib['height'] if i.isdigit())) + 50
+        else:
+            img = Image.open(machine_path)
+            width, height = img.size
     except:
         print("WARNING: Could not retrieve image dimensions")
         width, height = 800, 400
