@@ -34,7 +34,7 @@ class OutputManager(object):
     """Manager to handle different outputs (csv, database,... etc)
 
     """
-    def __init__(self, model=None, output_dir='', output_file='counts.csv'):
+    def __init__(self, model=None, output_dir='', output_file='counts.csv', log_file='log.txt'):
         """Initialize the output manager, specifying the model and the
         directory where outputs will be stored.
 
@@ -42,10 +42,12 @@ class OutputManager(object):
         self.model = model
         self.output_dir = output_dir
         self.output_file = output_file
+        self.log_file = log_file
 
         # database engine
         self.engine = None
-        self.counts_filename = None
+        self.counts_path = None
+        self.log_path = str(Path(self.output_dir, self.log_file))
         # We choose csv file by default
         self.output_type = 'csv'
         self.update_output_type()
@@ -82,8 +84,7 @@ class OutputManager(object):
             self.output_dir = self.output_dir.replace('/', '_')
 
         elif self.output_type == 'csv':
-            filename = self.output_file
-            self.counts_filename = str(Path(self.output_dir, filename))
+            self.counts_path = str(Path(self.output_dir, self.output_file))
         else:
             print('unknown output type !!!{]!!!')
 
@@ -92,8 +93,8 @@ class OutputManager(object):
 
         """
         if self.output_type == 'csv':
-            header = not exists(self.counts_filename)
-            with open(self.counts_filename, 'a') as f:
+            header = not exists(self.counts_path)
+            with open(self.counts_path, 'a') as f:
                 df.to_csv(f, header=header, index=False)
         elif self.output_type == 'database':
             df.to_sql(self.output_dir, self.engine, if_exists='append',
@@ -152,8 +153,8 @@ class AbstractSimulation(object):
         self.save_to_file = save_to_file
         self.load_from_file = load_from_file
 
-        filename = 'counts.csv'
-        self.counts_filename = str(Path(output_dir, filename))
+        # filename = 'counts.csv'
+        # self.counts_path = str(Path(output_dir, filename))
 
     @abc.abstractmethod
     def evolve(self, steps: int = 1):
@@ -174,18 +175,18 @@ class AbstractSimulation(object):
         """
         pass
 
-    def update_csv_counts(self, df, header: bool = True, dparams: dict = {}):
+    def update_csv_counts(self, df, dparams: dict = {}):
         """Update the CSV recording of populations in each state. """
         if self.save_results:
             for name, value in dparams.items():
                 df.insert(0, name, value)
-            # with open(self.counts_filename, 'a') as f:
+            # with open(self.counts_path, 'a') as f:
             #     df.to_csv(f, header=header, index=False)
             self.output_manager.update_outputs(df=df)
 
-    def counts_to_csv(self):
-        """Record the counts into a CSV file."""
-        self.counts.to_csv(self.counts_filename, index=False)
+    # def counts_to_csv(self):
+    #     """Record the counts into a CSV file."""
+    #     self.counts.to_csv(self.counts_path, index=False)
 
 #   _____ _                 _       _   _
 #  / ____(_)               | |     | | (_)
@@ -219,11 +220,17 @@ class Simulation(AbstractSimulation):
         self.quiet = quiet
 
         del(others['target_agent_class'])
+        others['simu_id'] = self.simu_id
+        others['simulation'] = self
         self.agent = self.init_agent(**others) ##self.target_agent_class(**others)
 
         self.outputs_period = self.model.outputs[self.agent.level]['period']\
                                 if self.agent.level in self.model.outputs else 1
 
+    def log_path(self):
+        """Return the log path used by current simulation"""
+        return self.output_manager.log_path
+
     def init_agent(self, **others):
         """Create an agent from the target class."""
         toplevel_agent = self.target_agent_class(**others)
@@ -250,14 +257,12 @@ class Simulation(AbstractSimulation):
         else:
             progress = trange(self.steps, desc=f'[Run {self.simu_id}]',
                               bar_format='{l_bar}{bar}| {n_fmt}/{total_fmt}')
-        self.update_csv_counts(self.counts,
-                               header=not exists(self.counts_filename),
-                               dparams=dparams)
+        self.update_csv_counts(self.counts, dparams=dparams)
         for step in progress:
             self.agent.evolve()
             if step % self.outputs_period == 0:
                 self.update_csv_counts(self.counts,
-                                       header=not exists(self.counts_filename),
+                                       # header=not exists(self.counts_path),
                                        dparams=dparams)
 
         ## make main level agent run a "finalization" procedure if any
@@ -356,7 +361,7 @@ class MultiSimulation(AbstractSimulation):
                 self.d_simu[simu_id] = simu
 
             # if update:
-            #     header = not exists(self.counts_filename)
+            #     header = not exists(self.counts_path)
             #     self.update_csv_counts(simu.counts, header=header, dparams=dparams)
 
     @property
@@ -431,7 +436,7 @@ class SensitivitySimulation(AbstractSimulation):
                 self.d_multi[multi_id] = multi
 
             # if update:
-            #     header = not exists(self.counts_filename)
+            #     header = not exists(self.counts_path)
 
             #     counts = multi.counts
             #     for name, value in self.d_scenario[multi_id].items():
