@@ -1,8 +1,11 @@
 """.. module:: emulsion.tools.state
 
-Tools aimed at handling state variables, either using a special
-dictionary (``StateVarDict``), or using a special descriptor
-(StateVar) in association with a decorator (@statevar).
+This module defines:
+
+- a class ``StateVarDict`` which is a special auto-referential
+  dictionary to handle state variables
+- a class ``EmulsionEnum`` which is a special kind of enumeration
+  intended to handle states from state machines
 
 """
 
@@ -13,31 +16,30 @@ dictionary (``StateVarDict``), or using a special descriptor
 # Contributors and contact:
 # -------------------------
 # 
-#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Sébastien Picault (sebastien.picault@inrae.fr)
 #     - Yu-Lin Huang
 #     - Vianney Sicard
 #     - Sandie Arnoux
 #     - Gaël Beaunée
-#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+#     - Pauline Ezanno (pauline.ezanno@inrae.fr)
 # 
-#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
-#     Nantes CS 44307 CEDEX, France
+#     INRAE, Oniris, BIOEPAR, 44300, Nantes, France
 # 
 # 
 # How to cite:
 # ------------
 # 
-#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
-#     Sustainability of Complex Epidemiological Models through a Generic
-#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
-#     International Joint Conference on Artificial Intelligence (IJCAI),
-#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+#     S. Picault, Y.-L. Huang, V. Sicard, S. Arnoux, G. Beaunée,
+#     P. Ezanno (2019). "EMULSION: Transparent and flexible multiscale
+#     stochastic models in human, animal and plant epidemiology", PLoS
+#     Computational Biology 15(9): e1007342. DOI:
+#     10.1371/journal.pcbi.1007342
 # 
 # 
 # License:
 # --------
 # 
-#    Copyright 2016 INRA and Univ. Lille
+#    Copyright 2016 INRAE and Univ. Lille
 # 
 #    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
 # 
@@ -62,23 +64,16 @@ from   functools                  import total_ordering
 
 
 class StateVarDict(dict):
-    """A special dictionary aimed at handling the 'State Variables' of
-    agents in Emulsion models. In addition to the classical dict
+    """A special dictionary aimed at handling the state variables of
+    agents in EMULSION models. In addition to the classical dict
     key-based access, it provides an attribute-like access syntax.
 
-    Example::
-
-        s = StateVarDict(age=10, sick=True)
-        s['age'] += 1
-        s.sick = False
-        s.new_property = 'Wow !'
-
-    This class is used in Emulsion to store individual agent
-    properties. Such properties include those defined automatically
-    and used by the Emulsion engine, such as the values of the states
-    for each state machine, the current time step, or "hidden" values
-    such as the time spent in the current state for each state
-    machine. They also include user-defined attributes (e.g. age,
+    This class is used in EMULSION to store agent properties. Such
+    properties include those defined automatically and used by the
+    EMULSION engine, such as the values of the states for each state
+    machine, the current time step, or "hidden" values such as the
+    time spent in the current state for each state machine. They are
+    also used to include user-defined attributes (e.g. age,
     weight...).
 
     When searching for a model ``statevar``, the engine tries first to
@@ -87,6 +82,16 @@ class StateVarDict(dict):
     agent's ``statevar`` attribute; finally, the search continues in
     the agent's host (if any).
 
+    Example
+    -------
+
+    .. code-block:: python
+
+        s = StateVarDict(age=10, sick=True)
+        s['age'] += 1
+        s.sick = False
+        s.new_property = 'Wow !'
+
     """
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -96,21 +101,35 @@ class StateVarDict(dict):
 
 @total_ordering
 class EmulsionEnum(Enum):
-    """This class represents enumerations for Emulsion. They are endowed
-    with some special features:
+    """This class represents enumerations for states of state machines in
+    EMULSION. They are endowed with some special features:
 
-     #. They provide total ordering between items (based on ``__lt__``
+    #. They provide total ordering between items (based on ``__lt__``
        and ``__eq__`` methods).
-     #. A comparison with ``None`` is provided (always greater than ``None``).
-     #. Other features will be developed soon.
+    #. A comparison with ``None`` is provided (any state is always
+       greater than ``None``).
+    #. Other features will be developed soon.
 
     """
+
     def __lt__(self, other):
         """Return a less-than comparison for enumeration items. An enum item
         is always greater than ``None``and than any other item of
         another enumeration. Then, the ``value`` attribute of the item
         is used.
 
+        Parameters
+        ----------
+        other: EmulsionEnum
+            Another state from the same enumeration (or ``None``)
+
+        Returns
+        -------
+        boolean:
+            True if the current state is less than *other*
+            (i.e. defined before *other* in the list of states), False
+            otherwise (including the case where *other* is ``None``).
+
         """
         return False if other is None or other.__class__ != self.__class__\
           else self.value < other.value
@@ -121,6 +140,18 @@ class EmulsionEnum(Enum):
         enumeration. Then, the ``value`` attribute of the item is
         used.
 
+        Parameters
+        ----------
+        other: EmulsionEnum
+            Another state from the same enumeration (or ``None``)
+
+        Returns
+        -------
+        boolean:
+            True if the current state is equal to *other* (having the
+            same value), False otherwise (including the case where
+            *other* is ``None``).
+
         """
         return False if other is None or other.__class__ != self.__class__\
           else self.value == other.value
